@@ -194,22 +194,35 @@ dir_is_temp_dir (const char *dir) {
 }
 
 
-/* Check whether the path_src is contained in path_dest */
+/* Check whether the dirname is contained in filename */
 gboolean
-path_in_path (const char  *path_src,
-	      const char  *path_dest)
+path_in_path (const char *dirname,
+	      const char *filename)
 {
-	int path_src_l, path_dest_l;
+	int dirname_l, filename_l, separator_position;
 
-	if ((path_src == NULL) || (path_dest == NULL))
+	if ((dirname == NULL) || (filename == NULL))
 		return FALSE;
 
-	path_src_l = strlen (path_src);
-	path_dest_l = strlen (path_dest);
+	dirname_l = strlen (dirname);
+	filename_l = strlen (filename);
 
-	return ((path_dest_l > path_src_l)
-		&& (strncmp (path_src, path_dest, path_src_l) == 0)
-		&& ((path_src_l == 1) || (path_dest[path_src_l] == '/')));
+	if ((dirname_l == filename_l + 1)
+	     && (dirname[dirname_l - 1] == '/'))
+		return FALSE;
+
+	if ((filename_l == dirname_l + 1)
+	     && (filename[filename_l - 1] == '/'))
+		return FALSE;
+
+	if (dirname[dirname_l - 1] == '/')
+		separator_position = dirname_l - 1;
+	else
+		separator_position = dirname_l;
+
+	return ((filename_l > dirname_l)
+		&& (strncmp (dirname, filename, dirname_l) == 0)
+		&& (filename[separator_position] == '/'));
 }
 
 
@@ -418,50 +431,50 @@ dir_name_from_path (const gchar *path)
 gchar *
 remove_level_from_path (const gchar *path)
 {
-        int         p;
-        const char *ptr = path;
-        char       *new_path;
+	int         p;
+	const char *ptr = path;
+	char       *new_path;
 
-        if (! path)
-                return NULL;
+	if (! path)
+		return NULL;
 
-        p = strlen (path) - 1;
-        if (p < 0)
-                return NULL;
+	p = strlen (path) - 1;
+	if (p < 0)
+		return NULL;
 
-        while ((p > 0) && (ptr[p] != '/'))
-                p--;
-        if ((p == 0) && (ptr[p] == '/'))
-                p++;
-        new_path = g_strndup (path, (guint)p);
+	while ((p > 0) && (ptr[p] != '/'))
+		p--;
+	if ((p == 0) && (ptr[p] == '/'))
+		p++;
+	new_path = g_strndup (path, (guint)p);
 
-        return new_path;
+	return new_path;
 }
 
 
 gchar *
 remove_extension_from_path (const gchar *path)
 {
-        int         len;
-        int         p;
-        const char *ptr = path;
-        char       *new_path;
+	int         len;
+	int         p;
+	const char *ptr = path;
+	char       *new_path;
 
-        if (! path)
-                return NULL;
+	if (! path)
+		return NULL;
 
-        len = strlen (path);
-        if (len == 1)
-                return g_strdup (path);
+	len = strlen (path);
+	if (len == 1)
+		return g_strdup (path);
 
-        p = len - 1;
-        while ((p > 0) && (ptr[p] != '.'))
-                p--;
-        if (p == 0)
-                p = len;
-        new_path = g_strndup (path, (guint) p);
+	p = len - 1;
+	while ((p > 0) && (ptr[p] != '.'))
+		p--;
+	if (p == 0)
+		p = len;
+	new_path = g_strndup (path, (guint) p);
 
-        return new_path;
+	return new_path;
 }
 
 
@@ -880,7 +893,7 @@ check_permissions (const char *path,
 gboolean
 is_program_in_path (const char *filename)
 {
-        char *str;
+	char *str;
 	char *value;
 	int   result = FALSE;
 
@@ -890,15 +903,15 @@ is_program_in_path (const char *filename)
 		return result;
 	}
 
-        str = g_find_program_in_path (filename);
-        if (str != NULL) {
-                g_free (str);
+	str = g_find_program_in_path (filename);
+	if (str != NULL) {
+		g_free (str);
 		result = TRUE;
-        }
+	}
 
 	g_hash_table_insert (programs_cache,
 			     g_strdup (filename),
 			     result ? "1" : "0");
 
-        return result;
+	return result;
 }
