@@ -26,8 +26,6 @@
 #include <unistd.h>
 
 #include <glib.h>
-#include <libgnomevfs/gnome-vfs-types.h>
-#include <libgnomevfs/gnome-vfs-mime.h>
 
 #include "file-data.h"
 #include "file-utils.h"
@@ -104,6 +102,7 @@ unstuff_is_shit_with_filenames (const char *orig)
 	return filename;
 }
 	
+	
 static void
 process_line (char     *line, 
 	      gpointer  data)
@@ -165,10 +164,11 @@ process_line (char     *line,
 
 	fdata = file_data_new ();
 	fdata->full_path = filename;
+	fdata->original_path = filename;
 	fdata->link = NULL;
 	fdata->name = g_strdup (file_name_from_path (fdata->full_path));
 	fdata->path = remove_level_from_path (fdata->full_path);
-	fdata->type = gnome_vfs_mime_type_from_name_or_default (fdata->name, GNOME_VFS_MIME_TYPE_UNKNOWN);
+
 	fdata->size = 0;
 	fdata->modified = time (NULL);
 
@@ -180,7 +180,8 @@ process_line (char     *line,
 
 
 static void
-fr_command_unstuff_list (FRCommand *comm)
+fr_command_unstuff_list (FRCommand  *comm,
+			 const char *password)
 {
 	char *arg, *path;
 	char *filename;
@@ -226,7 +227,6 @@ fr_command_unstuff_extract (FRCommand  *comm,
 	char  *filename;
 
 	fr_process_begin_command (comm->process, "unstuff");
-	fr_process_add_arg (comm->process, "x");
 
 	if (dest_dir != NULL) {
 		char *e_dest_dir = fr_command_escape (comm, dest_dir);
